@@ -9,7 +9,7 @@ try:
         import pkg_resources
 
         __version__ = pkg_resources.get_distribution(__package__ or __name__).version
-except Exception:
+except BaseException:
     pass
 
 import google.protobuf.message
@@ -17,11 +17,11 @@ import google.protobuf.message
 ProtobufMessage = google.protobuf.message.Message
 
 import nibiru.exceptions  # noqa
-import nibiru.msg  # noqa
 import nibiru.pytypes  # noqa
 from nibiru.grpc_client import GrpcClient  # noqa
-from nibiru.network import Network  # noqa
+from nibiru.msg import Msg  # noqa
+from nibiru.pytypes import Network  # noqa
 from nibiru.pytypes import Coin, Direction, PoolAsset, Side, TxConfig, TxType  # noqa
 from nibiru.sdk import Sdk  # noqa
-from nibiru.transaction import Transaction  # noqa
+from nibiru.tx import Transaction  # noqa
 from nibiru.wallet import Address, PrivateKey, PublicKey  # noqa
